// c_render.h: interface for the c_render class.
//
//////////////////////////////////////////////////////////////////////


/*
	Condor_render - submits maya scene file to be rendered under condor

	Copyright 2001 Colin Stolley
*/

// constants

#define CONDOR_SUBMIT_FILE "condor_render.sub"  // name of condor submit file to be generated

// the following path is used if the env variable CONDOR_RENDER_HDR is not set


#define JUNK_CHARS "\n\'\"();\t " // chars to be stripped from ends of path strings
#define WHITESPACE_CHARS "\n\t \r" // whitespace to be ignored

// things to parse for out of maya ascii files
#define MA_TEXTURE_TAG "setAttr \".ftn\" -type \"string\"" 
#define MA_FILEREF_TAG "file -r "

// things to parse for binary files (except they're wrong since I haven't figured them out yet)
#define MB_TEXTURE_TAG "ftn"
#define MB_FILEREF_TAG "FREFFREF"

#define MAX_BUFFER_SIZE 40960 // maximum size for statement buffer

#ifdef WIN32 //Windows
#define getcwd _getcwd
#define MAXPATHLEN _MAX_PATH
#include <direct.h>
#define PATH_SEPARATOR  '\\'
#define CONDOR_SUBMIT_HDR "\\condor_render.hdr" // name of file that gets prepended to submit file
#define CONDOR_RENDER_PATH_DEFAULT "C:\\condor_render" // default path to condor_render files including header file

#else // Unix
#include <unistd.h>
#include <sys/param.h>
#define PATH_SEPARATOR '/'
#define CONDOR_SUBMIT_HDR "/condor_render.hdr" // name of file that gets prepended to submit file
#define CONDOR_RENDER_PATH_DEFAULT "~/condor_render" // default path to condor_render files including header file

#endif

#include <stdio.h>

#include "mfstream.h"

#include <stdlib.h>
#include <string.h>
#include <ctype.h>
#include <sys/stat.h>

#include <vector>

using namespace std;

/*  we will eventually have the ref lists external so it's
	easier to find cyclical file references
vector <char*> Main_texture_refs;	// list of texture files that are used by this maya file
int Main_texture_ref_cnt;
vector <char*> Main_file_refs; // 
int Main_file_ref_cnt;

*/

int fileCopy( char* source, char* dest );
char* getFilename( char* path );

void changeFixedLengthString( char* oldstr, char* newstr ); // copies newstr to oldstr, pads with spaces

class c_render {

public:

	c_render(char *mayafile); 
	~c_render();
	void init(); // initializes all of the class members

	char* mayafile; // path of maya scene file to be rendered
	
	// make this private
	char buf[MAX_BUFFER_SIZE]; // class buffer used to store statements read from scene file
		
	enum fileType { ASCII, BINARY };	
	fileType mayaFileType;	// indicates the format of the maya file

	vector <char*> texture_refs;	// list of texture files that are used by this maya file
	int texture_ref_cnt;			// file count
	vector <char*> file_refs;		// list of file refs
	int file_ref_cnt;				// file count

	vector <char*> transfer_files;	// list of files to transfer with job (combination of texture_refs and file_refs
									// but the paths are all cwd and the filenames are appropriately renamed (e.g. ~file)
	int transfer_files_cnt;			// file count

	
	c_render();  //default constructor
	
	void usage(); // prints help screen to stdout

	int parseCommandArgs( int argc, char *argv[] ); // parses command line switches
	void verifyArguments();// verifies what parseCommandArgs() got

	int readNextLine();// public interface - reads next line statement from maya file (terminated by a ';')
	
	char* strrchrq( const char* str, int delim );// like strrchr but ignores stuff inside "" qualifiers

	int processFile();	// main method that kicks off all the other methods to process the maya file
	
	int printDeps(); // prints dependency list

	void addToTextureList( char* str ) { 
		fixPathSeparators( str );
		texture_refs.push_back( str );
		texture_ref_cnt++;
	};
	void addToFilerefList( char* str ) { 	
		fixPathSeparators( str );
		file_refs.push_back( str );
		file_ref_cnt++;
	};
	void addToTransferFileList( char* str ) {
		fixPathSeparators( str );
		transfer_files.push_back( str );
		transfer_files_cnt++;	
	};

	void writeToTranslationLog( char* str1, char* str2 ) { };
	void writeToSubmitFile( char* str );
	bool isPathSeparator( char c ) { return (c == '\\' || c == '/' ); };
	int submitToCondor();  // call condor_submit and fire away the jobs


private:
	
	//================
	// class variables
	//================
	double startFrame; // frame to start rendering on
	double endFrame; // frame to stop rendering on
	int jobSize;  // number of frames per condor job
	double byFrame; // step by frame for rendered sequence

	char* output_file_name;
	char* Condor_submit_hdr_file;

	mfstream* MFS; // Maya File Stream 

	//=======
	//methods
	//=======
	int openSceneFile(); // init mayaFilePtr, determine if mayafile is ascii or binary

	int openBinaryFile(); // specific crap that has to be done for binary files
		
	// getNextStatment() copies next statement in maya file to buf and returns 1, 
	// or else returns 0 if EOF is reached. Prints warning to stderr if buffer
	// overflow occurs.
	int getNextStatement(); // for parsing ascii files
	int getNextString(); // for parsing binary files
	
	// parses string in buf for texture references, if any are found they are appended to texture_refs
	int grabTextureRefs_MA();
	int grabTextureRefs_MB(); 

	// parses string in buf for file references, if any are found they are appended to file_refs
	int grabFileRefs_MA();
	int grabFileRefs_MB(); 

	
	int isRelativePath( const char* path );  // returns true if path is relative
	
	// canonicalizePath() takes a pointer to a path string, canonicalizes it and 
	// returns a new pointer to a new string. A successful operation returns non-zero value.
	int canonicalizePath( const char* cwd, char* &path );

	int fixFilePathForMaya( char* &path ); // massages file path as it comes out of maya scene file
	void fixPathSeparators( char* &path ); // homogenizes path separators
	
	int getParentDirectory( char* path );  // peels off rightmost name from path string
	//char* getFilename( const char* path );

	int parseDeps(); // build lists of texture and file refs (recursively)

	// returns non-zero if character can safely be stripped from a string
	// returns 0 if character should be saved
	int isJunkCharacter( int c ) {
		return (int) strchr( JUNK_CHARS, c );
	};

	int isWhiteSpace ( int c ) {
		return (int) strchr( WHITESPACE_CHARS, c );
	};

	void trim( char* str );  // removes leading and trailing whitespace from str

	int buildSubmitFile(); // write Condor submit file 
};

